import java.awt.*;

//visited color - 221,224,194

public class ChessBoard
{
	Cell iCells[][];//board-matrix
	int iStartX, iStartY;//start offset of board from origin
	public static final int BOARD_SIZE = 248;//fixed board-size
	int iInterval;//width/height of each cell - being a square both length and breadth
				  //values are the same
	int iStartRow, iStartCol;
	Point ptKnightPos;
	
	public static final int GAME_OVER = 1;
	public static final int GAME_ON = 2;
	public static final int KNIGHT = (int)'K';
		
	private ChessBoard()
	{
	}
	
	public ChessBoard( Graphics g, int x, int y, int iStartRow, int iStartCol )//start offset of board from origin
	{
		iCells = new Cell[8][8];
		
		iStartX = x;//store offset for later use
		iStartY = y;
		
		iInterval = BOARD_SIZE / 8;//calculate width of each cell
		
		initBoard( g, iStartRow, iStartCol );
		
		this.iStartRow = iStartRow;
		this.iStartCol = iStartCol;
	}
	
	public void initBoard( Graphics g, int iStartRow, int iStartCol )
	{
		for( int i = 0 ; i < 8 ; ++i )//initialize board-matrix
		{
			for( int j = 0 ; j < 8 ; ++j )
			{
				iCells[i][j] = new Cell();
				setValue( g, i, j, 0 );
			}
		}
		
		ptKnightPos = new Point( iStartRow, iStartCol );
		iCells[iStartRow][iStartCol].bTouched = true;
		
		setValue( g, iStartRow, iStartCol, (int)'*' );
	}
	
	public void drawBoard( Graphics g )
	{
		g.drawRect( iStartX, iStartY, BOARD_SIZE, BOARD_SIZE );//draw outline
		
		int iX = iStartX + iInterval;//start from top-left corner and move right
		
		for( int i = 0 ; i < 7 ; ++i )//there will be 7 lines inside the square
		{
			g.drawLine( iX, iStartY, iX, (iStartY + BOARD_SIZE) );//draw vertical line
			iX += iInterval;//move to next point in square
		}
		
		int iY = iStartY + iInterval;//start from top-left corner and move down
		
		for( int i = 0 ; i < 7 ; ++i )//there will be 7 lines inside the square
		{
			g.drawLine( iStartX, iY, (iStartX + BOARD_SIZE), iY);//draw horizntal line
			iY += iInterval;//move to next point in square
		}
	}
	
	public Point getCoords( int iRow, int iCol )
	{
		//	Converts a given co-ordinate in terms of the board-matrix into
		//	screen co-ordinates.  Returns a reference to a point object
		//	that is the top-left point of the individual cell.
		
		if( ( iRow > 7 ) || ( iCol > 7 ) )
			return null;
		
		//simply multiply by the board-matrix value and adjust
		//for start offset
		int x = ( iInterval * iRow ) + iStartX;
		int y = ( iInterval * iCol ) + iStartY;
		
		return new Point( x, y );
	}
	
	public void setValue( Graphics g, int iRow, int iCol, int iValue )
	{
		//	Sets a value in the board matrix and also updates the display.
		
		if( ( iRow > 7 ) || ( iCol > 7 ) )
			return;
		
		Point ptCoord = getCoords( iRow, iCol );
		Color clr = g.getColor();
						
		iCells[iRow][iCol].iValue = iValue;//set value in board-matrix
		
		if( iCells[iRow][iCol].bTouched )
		{
			if( ( iRow == iStartRow ) && ( iCol == iStartCol ) )
				g.setColor( new Color( 134, 202, 227 ) );
			else
				g.setColor( new Color( 221, 224, 194 ) );

			g.fillRect( (ptCoord.x + 2), (ptCoord.y + 2), (iInterval - 2),
						(iInterval - 2) );
			
			g.setColor( clr );
		}

		showChar( g, ptCoord, iCells[iRow][iCol].iValue );//now show the new value
	}
	
	public int getValue( int iRow, int iCol )
	{
		//	Returns the value in the board at a particular cell
		
		if( ( iRow > 7 ) || ( iCol > 7 ) )
			return -1;
		
		return iCells[iRow][iCol].iValue;
	}
	
	public void printValues( Graphics g )
	{
		//shows all the values in the matrix onto the screen
		
		int iRow, iCol;
		
		for( iRow = 0 ; iRow < 8 ; ++iRow )
		{
			for( iCol = 0 ; iCol < 8 ; ++iCol )
			{
				Point ptCoord = getCoords( iRow, iCol );
				setValue( g, iRow, iCol, iCells[iRow][iCol].iValue );
			}
		}
	}
	
	private void showChar( Graphics g, Point ptCoord, int ch )
	{
		//	This method does the actual showing of each individual
		//	character.
		
		FontMetrics fm = g.getFontMetrics();//required for adjusting for width of
											//each character
		
		int iCharWidth = fm.charWidth( ch );//get the width of the character
		int iCharHeight = fm.getHeight();//get its height
		
		ptCoord.x += (iInterval - iCharWidth) / 2;//center it in the cell along x-axis
		ptCoord.y += (iInterval - iCharHeight);//center it along y-axis
		
		//display the character
		g.drawString( String.valueOf( (char)ch ),
			ptCoord.x, ptCoord.y );
	}

	public int[] removeTouchedMoves( int[] iMoves )
	{
		int iNewMoves[] = new int[8];
		int i, j;
		
		for( i = 0 ; i < 8 ; ++i )
			iNewMoves[i] = -1;
		
		for( i = 0, j = 0 ; ( i < 8 ) && ( iMoves[i] != -1 ) ; ++i )
		{
			int iIndex = iMoves[i];
			int iRow = ptKnightPos.x;
			int iCol = ptKnightPos.y;
			
			iRow += KnightMoves.iHorizontalMove[iIndex];
			iCol += KnightMoves.iVerticalMove[iIndex];
			
			if( !( iCells[iRow][iCol].bTouched ) )
				iNewMoves[j++] = iMoves[i];
		}
		
		return iNewMoves;
	}
		
	public int makeMove( Graphics g )
	{
		int iMoves[] = KnightMoves.enumerateMoves( ptKnightPos );
		
		if( iMoves == null || iMoves[0] == -1 )
		{
			System.out.println( "RETURNING...enumerateMoves FAILED." );
			return GAME_OVER;
		}
		
		iMoves = removeTouchedMoves( iMoves );
		
		if( iMoves[0] == -1 )
			return GAME_OVER;
		
		int iAptMove = AccessibilityMatrix.getAptMove( iMoves, ptKnightPos );

		if( iAptMove == -1 )
			return GAME_OVER;
					
		int iIndex = iAptMove;
		
		setValue( g, ptKnightPos.x, ptKnightPos.y, (int)'*' );
		
		ptKnightPos.x += KnightMoves.iHorizontalMove[iIndex];
		ptKnightPos.y += KnightMoves.iVerticalMove[iIndex];
		
		iCells[ptKnightPos.x][ptKnightPos.y].bTouched = true;
		
		setValue( g, ptKnightPos.x, ptKnightPos.y, KNIGHT );
		
		return GAME_ON;
	}
}
