import java.awt.*;
import java.applet.*;
import java.awt.event.*;

public class KnightsTour extends Applet implements Runnable, ActionListener
{
	ChessBoard board;
	TextField txtInterval;
	Label lblInterval;
	Button btnStart;
	Thread thrdGame;
	int iInterval;
	int iFullGames;

	public void init()
	{
		board = null;
		thrdGame = null;
		KnightMoves.init();
		AccessibilityMatrix.init();
		
		lblInterval = new Label( "Interval between moves (millisecs) : " );
		add( lblInterval );
		
		txtInterval = new TextField( 6 );
		add( txtInterval );
		txtInterval.addActionListener( this );
		
		btnStart = new Button( "Start" );
		add( btnStart );
		btnStart.addActionListener( this );
		
		iInterval = 1250;
		txtInterval.setText( String.valueOf( iInterval ) );
		
		iFullGames = 0;
	}
		
	public void paint( Graphics g )
	{
		if( board != null )
		{
			board.drawBoard( g );
			board.printValues( g );
		}
		else
		{
			new ChessBoard( g, 18, 30, 0, 0 ).drawBoard( g );
			showStatus( "Click \"Start\" to start the game." );
		}
	}
	
	public void run()
	{
		int iResult = ChessBoard.GAME_ON;
		Graphics g = getGraphics();
		Color clrErase = getBackground();
		int i, j;
		int iGameCount = 0;
				
		for( i = 0 ; i < 8 ; ++i )
		{
			for( j = 0 ; j < 8 ; ++j )
			{
				int iMoveCount = 1;
				board = new ChessBoard( g, 18, 30, i, j );
				repaint();
				
				showStatus( "Game No. : " + String.valueOf( ++iGameCount ) );
								
				do
				{
					iResult = board.makeMove( g );
					
					if( iResult != ChessBoard.GAME_OVER )
						++iMoveCount;
					
					try
					{
						thrdGame.sleep( iInterval );
					}
					catch( InterruptedException e )
					{
						e.printStackTrace();
					}
						
				}while( iResult != ChessBoard.GAME_OVER );
				
				if( iMoveCount == 64 )
				{
					System.out.println( "Got a game at Row = " + i + ", Col = " + j + "." );
					
					showStatus( "There, that was a full game! Have got " + ( ++iFullGames ) + " full game(s) so far." );
					try
					{
						thrdGame.sleep( 3000 );
					}
					catch( InterruptedException ex )
					{
						ex.printStackTrace();
					}
				}
				iMoveCount = 1;
			}
		}
		
		showStatus( "Game Over!!  Look at the console for the results." );
	}
	
	public void stop()
	{
		thrdGame.stop();
	}
	
	public void actionPerformed( ActionEvent e )
	{
		if( e.getActionCommand() == "Start" )
		{
			thrdGame = new Thread( this );
			thrdGame.start();
			
			btnStart.setEnabled( false );
			
			return;
		}
		
		thrdGame.suspend();
		
		try
		{
			iInterval = Integer.parseInt( txtInterval.getText() );
		}
		catch( NumberFormatException ex )
		{
			showStatus( "Please enter a valid number." );
		}
		
		thrdGame.resume();
	}
}